package test.portlet.producer;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import test.portlet.common.TestEvent;

import com.vaadin.Application;
import com.vaadin.service.ApplicationContext.TransactionListener;
import com.vaadin.terminal.gwt.server.PortletApplicationContext2;

public class ProducerController {
    private Log log = LogFactory.getLog(this.getClass());
    private ProducerApplication application;
    private static TransactionListener tl;

    private static ThreadLocal<ProducerController> controller = new ThreadLocal<ProducerController>();

    public ProducerController(ProducerApplication application) {
        this.application = application;
    }

    public void init() {
        // get the controller from the application and set it for the initial phase, e.g.
        // for drawing the window
        ProducerController ctr = ((ProducerApplication) application).getController();
        controller.set(ctr);
        // create and add a transaction listener to be able to assign a controller instance to each
        // request from the client. On each transaction the controller must be set again. Otherwise
        // a NullPointerException will be thrown.
        tl = new WindowTransactionListener();
        this.application.getContext().addTransactionListener(tl);
        if (application.isInPortletContext()) {
            // do some stuff here
        } else {
            // not within portlet container. Return some simulation stuff!
            // return SimulationFactory.getTaskDescriptionList();
        }
    }

    public static ProducerController get() {
        return controller.get();
    }

    public Application getApplication() {
        return application;
    }

    public void removeTransactionListener() {
        ProducerController.get().getApplication().getContext().removeTransactionListener(tl);
    }

    @SuppressWarnings("serial")
    private final class WindowTransactionListener implements TransactionListener {

        @Override
        public void transactionStart(Application application, Object transactionData) {
            if (application instanceof ProducerApplication) {
                ProducerController ctr = ((ProducerApplication) application).getController();
                controller.set(ctr);
            } else {
                // This should not happen
                controller.set(null);
            }
        }

        @Override
        public void transactionEnd(Application application, Object transactionData) {
            if (application instanceof ProducerApplication) {
                controller.remove();
                // do stuff here
            }
        }
    }

    public void updateContent(int nr) {
        application.getMainWindow().updateContent(nr);
    }

    public void sendEvent(int clickNr) {
        if (application.isInPortletContext()) {
            PortletApplicationContext2 ctx = (PortletApplicationContext2) application.getContext();

            TestEvent event = new TestEvent("producer", "consumer", clickNr);
            ctx.sendPortletEvent(application.getMainWindow(), new QName("http://www.test.com/test",
                    "event"), event);
            log.debug("send portlet event: " + event);
        } 
        updateContent(clickNr);
    }

}
